@@ -6,7 +6,7 @@
 #include "cryptsetup-token.h"
 #include "cryptsetup-token-util.h"
 #include "hexdecoct.h"
-#include "json.h"
+#include "json-util.h"
 #include "luks2-tpm2.h"
 #include "memory-util.h"
 #include "strv.h"
@@ -42,9 +42,9 @@ _public_ int cryptsetup_token_open_pin(
                 void *usrptr /* plugin defined parameter passed to crypt_activate_by_token*() API */) {
 
         _cleanup_(erase_and_freep) char *base64_encoded = NULL, *pin_string = NULL;
-        _cleanup_(iovec_done) struct iovec blob = {}, pubkey = {}, policy_hash = {}, salt = {}, srk = {}, pcrlock_nv = {};
+        _cleanup_(iovec_done) struct iovec pubkey = {}, salt = {}, srk = {}, pcrlock_nv = {};
         _cleanup_(iovec_done_erase) struct iovec decrypted_key = {};
-        _cleanup_(json_variant_unrefp) JsonVariant *v = NULL;
+        _cleanup_(sd_json_variant_unrefp) sd_json_variant *v = NULL;
         uint32_t hash_pcr_mask, pubkey_pcr_mask;
         systemd_tpm2_plugin_params params = {
                 .search_pcr_mask = UINT32_MAX
@@ -72,10 +72,15 @@ _public_ int cryptsetup_token_open_pin(
         if (usrptr)
                 params = *(systemd_tpm2_plugin_params *)usrptr;
 
-        r = json_parse(json, 0, &v, NULL, NULL);
+        r = sd_json_parse(json, 0, &v, NULL, NULL);
         if (r < 0)
                 return crypt_log_debug_errno(cd, r, "Failed to parse token JSON data: %m");
 
+        struct iovec *blobs = NULL, *policy_hash = NULL;
+        size_t n_blobs = 0, n_policy_hash = 0;
+        CLEANUP_ARRAY(blobs, n_blobs, iovec_array_free);
+        CLEANUP_ARRAY(policy_hash, n_policy_hash, iovec_array_free);
+
         r = tpm2_parse_luks2_json(
                         v,
                         /* ret_keyslot= */ NULL,
@@ -84,8 +89,10 @@ _public_ int cryptsetup_token_open_pin(
                         &pubkey,
                         &pubkey_pcr_mask,
                         &primary_alg,
-                        &blob,
+                        &blobs,
+                        &n_blobs,
                         &policy_hash,
+                        &n_policy_hash,
                         &salt,
                         &srk,
                         &pcrlock_nv,
@@ -106,8 +113,10 @@ _public_ int cryptsetup_token_open_pin(
                         pin_string,
                         params.pcrlock_path,
                         primary_alg,
-                        &blob,
-                        &policy_hash,
+                        blobs,
+                        n_blobs,
+                        policy_hash,
+                        n_policy_hash,
                         &salt,
                         &srk,
                         &pcrlock_nv,
@@ -167,9 +176,9 @@ _public_ void cryptsetup_token_dump(
                 struct crypt_device *cd /* is always LUKS2 context */,
                 const char *json /* validated 'systemd-tpm2' token if cryptsetup_token_validate is defined */) {
 
-        _cleanup_free_ char *hash_pcrs_str = NULL, *pubkey_pcrs_str = NULL, *blob_str = NULL, *policy_hash_str = NULL, *pubkey_str = NULL;
-        _cleanup_(iovec_done) struct iovec blob = {}, pubkey = {}, policy_hash = {}, salt = {}, srk = {}, pcrlock_nv = {};
-        _cleanup_(json_variant_unrefp) JsonVariant *v = NULL;
+        _cleanup_free_ char *hash_pcrs_str = NULL, *pubkey_pcrs_str = NULL, *pubkey_str = NULL;
+        _cleanup_(iovec_done) struct iovec pubkey = {}, salt = {}, srk = {}, pcrlock_nv = {};
+        _cleanup_(sd_json_variant_unrefp) sd_json_variant *v = NULL;
         uint32_t hash_pcr_mask, pubkey_pcr_mask;
         uint16_t pcr_bank, primary_alg;
         TPM2Flags flags = 0;
@@ -177,10 +186,15 @@ _public_ void cryptsetup_token_dump(
 
         assert(json);
 
-        r = json_parse(json, 0, &v, NULL, NULL);
+        r = sd_json_parse(json, 0, &v, NULL, NULL);
         if (r < 0)
                 return (void) crypt_log_debug_errno(cd, r, "Failed to parse " TOKEN_NAME " JSON object: %m");
 
+        struct iovec *blobs = NULL, *policy_hash = NULL;
+        size_t n_blobs = 0, n_policy_hash = 0;
+        CLEANUP_ARRAY(blobs, n_blobs, iovec_array_free);
+        CLEANUP_ARRAY(policy_hash, n_policy_hash, iovec_array_free);
+
         r = tpm2_parse_luks2_json(
                         v,
                         NULL,
@@ -189,8 +203,10 @@ _public_ void cryptsetup_token_dump(
                         &pubkey,
                         &pubkey_pcr_mask,
                         &primary_alg,
-                        &blob,
+                        &blobs,
+                        &n_blobs,
                         &policy_hash,
+                        &n_policy_hash,
                         &salt,
                         &srk,
                         &pcrlock_nv,
@@ -206,30 +222,40 @@ _public_ void cryptsetup_token_dump(
         if (!pubkey_pcrs_str)
                 return (void) crypt_log_debug_errno(cd, ENOMEM, "Cannot format PCR hash mask: %m");
 
-        r = crypt_dump_buffer_to_hex_string(blob.iov_base, blob.iov_len, &blob_str);
-        if (r < 0)
-                return (void) crypt_log_debug_errno(cd, r, "Cannot dump " TOKEN_NAME " content: %m");
-
         r = crypt_dump_buffer_to_hex_string(pubkey.iov_base, pubkey.iov_len, &pubkey_str);
         if (r < 0)
                 return (void) crypt_log_debug_errno(cd, r, "Cannot dump " TOKEN_NAME " content: %m");
 
-        r = crypt_dump_buffer_to_hex_string(policy_hash.iov_base, policy_hash.iov_len, &policy_hash_str);
-        if (r < 0)
-                return (void) crypt_log_debug_errno(cd, r, "Cannot dump " TOKEN_NAME " content: %m");
-
         crypt_log(cd, "\ttpm2-hash-pcrs:   %s\n", strna(hash_pcrs_str));
         crypt_log(cd, "\ttpm2-pcr-bank:    %s\n", strna(tpm2_hash_alg_to_string(pcr_bank)));
         crypt_log(cd, "\ttpm2-pubkey:" CRYPT_DUMP_LINE_SEP "%s\n", pubkey_str);
         crypt_log(cd, "\ttpm2-pubkey-pcrs: %s\n", strna(pubkey_pcrs_str));
         crypt_log(cd, "\ttpm2-primary-alg: %s\n", strna(tpm2_asym_alg_to_string(primary_alg)));
-        crypt_log(cd, "\ttpm2-blob:  %s\n", blob_str);
-        crypt_log(cd, "\ttpm2-policy-hash:" CRYPT_DUMP_LINE_SEP "%s\n", policy_hash_str);
         crypt_log(cd, "\ttpm2-pin:         %s\n", true_false(flags & TPM2_FLAGS_USE_PIN));
         crypt_log(cd, "\ttpm2-pcrlock:     %s\n", true_false(flags & TPM2_FLAGS_USE_PCRLOCK));
         crypt_log(cd, "\ttpm2-salt:        %s\n", true_false(iovec_is_set(&salt)));
         crypt_log(cd, "\ttpm2-srk:         %s\n", true_false(iovec_is_set(&srk)));
         crypt_log(cd, "\ttpm2-pcrlock-nv:  %s\n", true_false(iovec_is_set(&pcrlock_nv)));
+
+        FOREACH_ARRAY(p, policy_hash, n_policy_hash) {
+                _cleanup_free_ char *policy_hash_str = NULL;
+
+                r = crypt_dump_buffer_to_hex_string(p->iov_base, p->iov_len, &policy_hash_str);
+                if (r < 0)
+                        return (void) crypt_log_debug_errno(cd, r, "Cannot dump " TOKEN_NAME " content: %m");
+
+                crypt_log(cd, "\ttpm2-policy-hash:" CRYPT_DUMP_LINE_SEP "%s\n", policy_hash_str);
+        }
+
+        FOREACH_ARRAY(b, blobs, n_blobs) {
+                _cleanup_free_ char *blob_str = NULL;
+
+                r = crypt_dump_buffer_to_hex_string(b->iov_base, b->iov_len, &blob_str);
+                if (r < 0)
+                        return (void) crypt_log_debug_errno(cd, r, "Cannot dump " TOKEN_NAME " content: %m");
+
+                crypt_log(cd, "\ttpm2-blob:        %s\n", blob_str);
+        }
 }
 
 /*
@@ -243,17 +269,17 @@ _public_ int cryptsetup_token_validate(
                 const char *json /* contains valid 'type' and 'keyslots' fields. 'type' is 'systemd-tpm2' */) {
 
         int r;
-        JsonVariant *w, *e;
-        _cleanup_(json_variant_unrefp) JsonVariant *v = NULL;
+        sd_json_variant *w, *e;
+        _cleanup_(sd_json_variant_unrefp) sd_json_variant *v = NULL;
 
         assert(json);
 
-        r = json_parse(json, 0, &v, NULL, NULL);
+        r = sd_json_parse(json, 0, &v, NULL, NULL);
         if (r < 0)
                 return crypt_log_debug_errno(cd, r, "Could not parse " TOKEN_NAME " json object: %m");
 
-        w = json_variant_by_key(v, "tpm2-pcrs");
-        if (!w || !json_variant_is_array(w)) {
+        w = sd_json_variant_by_key(v, "tpm2-pcrs");
+        if (!w || !sd_json_variant_is_array(w)) {
                 crypt_log_debug(cd, "TPM2 token data lacks 'tpm2-pcrs' field.");
                 return 1;
         }
@@ -261,12 +287,12 @@ _public_ int cryptsetup_token_validate(
         JSON_VARIANT_ARRAY_FOREACH(e, w) {
                 uint64_t u;
 
-                if (!json_variant_is_number(e)) {
+                if (!sd_json_variant_is_number(e)) {
                         crypt_log_debug(cd, "TPM2 PCR is not a number.");
                         return 1;
                 }
 
-                u = json_variant_unsigned(e);
+                u = sd_json_variant_unsigned(e);
                 if (!TPM2_PCR_INDEX_VALID(u)) {
                         crypt_log_debug(cd, "TPM2 PCR number out of range.");
                         return 1;
@@ -275,61 +301,79 @@ _public_ int cryptsetup_token_validate(
 
         /* The bank field is optional, since it was added in systemd 250 only. Before the bank was hardcoded
          * to SHA256. */
-        w = json_variant_by_key(v, "tpm2-pcr-bank");
+        w = sd_json_variant_by_key(v, "tpm2-pcr-bank");
         if (w) {
                 /* The PCR bank field is optional */
 
-                if (!json_variant_is_string(w)) {
+                if (!sd_json_variant_is_string(w)) {
                         crypt_log_debug(cd, "TPM2 PCR bank is not a string.");
                         return 1;
                 }
 
-                if (tpm2_hash_alg_from_string(json_variant_string(w)) < 0) {
-                        crypt_log_debug(cd, "TPM2 PCR bank invalid or not supported: %s.", json_variant_string(w));
+                if (tpm2_hash_alg_from_string(sd_json_variant_string(w)) < 0) {
+                        crypt_log_debug(cd, "TPM2 PCR bank invalid or not supported: %s.", sd_json_variant_string(w));
                         return 1;
                 }
         }
 
         /* The primary key algorithm field is optional, since it was also added in systemd 250 only. Before
          * the algorithm was hardcoded to ECC. */
-        w = json_variant_by_key(v, "tpm2-primary-alg");
+        w = sd_json_variant_by_key(v, "tpm2-primary-alg");
         if (w) {
                 /* The primary key algorithm is optional */
 
-                if (!json_variant_is_string(w)) {
+                if (!sd_json_variant_is_string(w)) {
                         crypt_log_debug(cd, "TPM2 primary key algorithm is not a string.");
                         return 1;
                 }
 
-                if (tpm2_asym_alg_from_string(json_variant_string(w)) < 0) {
-                        crypt_log_debug(cd, "TPM2 primary key algorithm invalid or not supported: %s", json_variant_string(w));
+                if (tpm2_asym_alg_from_string(sd_json_variant_string(w)) < 0) {
+                        crypt_log_debug(cd, "TPM2 primary key algorithm invalid or not supported: %s", sd_json_variant_string(w));
                         return 1;
                 }
         }
 
-        w = json_variant_by_key(v, "tpm2-blob");
-        if (!w || !json_variant_is_string(w)) {
+        w = sd_json_variant_by_key(v, "tpm2-blob");
+        if (!w) {
                 crypt_log_debug(cd, "TPM2 token data lacks 'tpm2-blob' field.");
                 return 1;
         }
 
-        r = unbase64mem(json_variant_string(w), NULL, NULL);
-        if (r < 0)
-                return crypt_log_debug_errno(cd, r, "Invalid base64 data in 'tpm2-blob' field: %m");
+        if (sd_json_variant_is_array(w)) {
+                sd_json_variant *i;
+                JSON_VARIANT_ARRAY_FOREACH(i, w) {
+                        r = sd_json_variant_unbase64(i, /* ret= */ NULL, /* ret_size= */ NULL);
+                        if (r < 0)
+                                return crypt_log_debug_errno(cd, r, "Invalid base64 data in 'tpm2-blob' field: %m");
+                }
+        } else {
+                r = sd_json_variant_unbase64(w, /* ret= */ NULL, /* ret_size= */ NULL);
+                if (r < 0)
+                        return crypt_log_debug_errno(cd, r, "Invalid base64 data in 'tpm2-blob' field: %m");
+        }
 
-        w = json_variant_by_key(v, "tpm2-policy-hash");
-        if (!w || !json_variant_is_string(w)) {
+        w = sd_json_variant_by_key(v, "tpm2-policy-hash");
+        if (!w) {
                 crypt_log_debug(cd, "TPM2 token data lacks 'tpm2-policy-hash' field.");
                 return 1;
         }
 
-        r = unhexmem(json_variant_string(w), NULL, NULL);
-        if (r < 0)
-                return crypt_log_debug_errno(cd, r, "Invalid base64 data in 'tpm2-policy-hash' field: %m");
+        if (sd_json_variant_is_array(w)) {
+                sd_json_variant *i;
+                JSON_VARIANT_ARRAY_FOREACH(i, w) {
+                        r = sd_json_variant_unhex(i, /* ret= */ NULL, /* ret_size= */ NULL);
+                        if (r < 0)
+                                return crypt_log_debug_errno(cd, r, "Invalid hex data in 'tpm2-policy-hash' field: %m");
+                }
+        } else {
+                r = sd_json_variant_unhex(w, /* ret= */ NULL, /* ret_size= */ NULL);
+                if (r < 0)
+                        return crypt_log_debug_errno(cd, r, "Invalid hex data in 'tpm2-policy-hash' field: %m");
+        }
 
-        w = json_variant_by_key(v, "tpm2-pin");
+        w = sd_json_variant_by_key(v, "tpm2-pin");
         if (w) {
-                if (!json_variant_is_boolean(w)) {
+                if (!sd_json_variant_is_boolean(w)) {
                         crypt_log_debug(cd, "TPM2 PIN policy is not a boolean.");
                         return 1;
                 }
