@@ -2,6 +2,7 @@
 
 #include <errno.h>
 #include <limits.h>
+#include <linux/ipv6.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -9,6 +10,7 @@
 
 #include "alloc-util.h"
 #include "chase.h"
+#include "calendarspec.h"
 #include "conf-files.h"
 #include "conf-parser.h"
 #include "constants.h"
@@ -22,7 +24,8 @@
 #include "hash-funcs.h"
 #include "hostname-util.h"
 #include "id128-util.h"
-#include "in-addr-util.h"
+#include "in-addr-prefix-util.h"
+#include "ip-protocol-list.h"
 #include "log.h"
 #include "macro.h"
 #include "missing_network.h"
@@ -171,7 +174,7 @@ static int next_assignment(
 
 /* Parse a single logical line */
 static int parse_line(
-                const char* unit,
+                const char *unit,
                 const char *filename,
                 unsigned line,
                 const char *sections,
@@ -462,10 +465,6 @@ int hashmap_put_stats_by_path(Hashmap **stats_by_path, const char *path, const s
         assert(path);
         assert(st);
 
-        r = hashmap_ensure_allocated(stats_by_path, &path_hash_ops_free_free);
-        if (r < 0)
-                return r;
-
         st_copy = newdup(struct stat, st, 1);
         if (!st_copy)
                 return -ENOMEM;
@@ -474,7 +473,7 @@ int hashmap_put_stats_by_path(Hashmap **stats_by_path, const char *path, const s
         if (!path_copy)
                 return -ENOMEM;
 
-        r = hashmap_put(*stats_by_path, path_copy, st_copy);
+        r = hashmap_ensure_put(stats_by_path, &path_hash_ops_free_free, path_copy, st_copy);
         if (r < 0)
                 return r;
 
@@ -499,12 +498,12 @@ static int config_parse_many_files(
         _cleanup_ordered_hashmap_free_ OrderedHashmap *dropins = NULL;
         _cleanup_set_free_ Set *inodes = NULL;
         struct stat st;
-        int r;
+        int r, level = FLAGS_SET(flags, CONFIG_PARSE_WARN) ? LOG_WARNING : LOG_DEBUG;
 
         if (ret_stats_by_path) {
                 stats_by_path = hashmap_new(&path_hash_ops_free_free);
                 if (!stats_by_path)
-                        return -ENOMEM;
+                        return log_oom_full(level);
         }
 
         STRV_FOREACH(fn, files) {
@@ -515,14 +514,14 @@ static int config_parse_many_files(
                 if (r == -ENOENT)
                         continue;
                 if (r < 0)
-                        return r;
+                        return log_full_errno(level, r, "Failed to open %s: %m", *fn);
 
                 int fd = fileno(f);
 
                 r = ordered_hashmap_ensure_put(&dropins, &config_file_hash_ops_fclose, *fn, f);
                 if (r < 0) {
-                        assert(r != -EEXIST);
-                        return r;
+                        assert(r == -ENOMEM);
+                        return log_oom_full(level);
                 }
                 assert(r > 0);
                 TAKE_PTR(f);
@@ -532,14 +531,14 @@ static int config_parse_many_files(
 
                 _cleanup_free_ struct stat *st_dropin = new(struct stat, 1);
                 if (!st_dropin)
-                        return -ENOMEM;
+                        return log_oom_full(level);
 
                 if (fstat(fd, st_dropin) < 0)
-                        return -errno;
+                        return log_full_errno(level, errno, "Failed to stat %s: %m", *fn);
 
                 r = set_ensure_consume(&inodes, &inode_hash_ops, TAKE_PTR(st_dropin));
                 if (r < 0)
-                        return r;
+                        return log_oom_full(level);
         }
 
         /* First read the first found main config file. */
@@ -550,11 +549,11 @@ static int config_parse_many_files(
                 if (r == -ENOENT)
                         continue;
                 if (r < 0)
-                        return r;
+                        return log_full_errno(level, r, "Failed to open %s: %m", *fn);
 
                 if (inodes) {
                         if (fstat(fileno(f), &st) < 0)
-                                return -errno;
+                                return log_full_errno(level, errno, "Failed to stat %s: %m", *fn);
 
                         if (set_contains(inodes, &st)) {
                                 log_debug("%s: symlink to/symlinked as drop-in, will be read later.", *fn);
@@ -564,13 +563,13 @@ static int config_parse_many_files(
 
                 r = config_parse(/* unit= */ NULL, *fn, f, sections, lookup, table, flags, userdata, &st);
                 if (r < 0)
-                        return r;
+                        return r; /* config_parse() logs internally. */
                 assert(r > 0);
 
                 if (ret_stats_by_path) {
                         r = hashmap_put_stats_by_path(&stats_by_path, *fn, &st);
                         if (r < 0)
-                                return r;
+                                return log_full_errno(level, r, "Failed to save stats of %s: %m", *fn);
                 }
 
                 break;
@@ -583,13 +582,13 @@ static int config_parse_many_files(
         ORDERED_HASHMAP_FOREACH_KEY(f_dropin, path_dropin, dropins) {
                 r = config_parse(/* unit= */ NULL, path_dropin, f_dropin, sections, lookup, table, flags, userdata, &st);
                 if (r < 0)
-                        return r;
+                        return r; /* config_parse() logs internally. */
                 assert(r > 0);
 
                 if (ret_stats_by_path) {
                         r = hashmap_put_stats_by_path(&stats_by_path, path_dropin, &st);
                         if (r < 0)
-                                return r;
+                                return log_full_errno(level, r, "Failed to save stats of %s: %m", path_dropin);
                 }
         }
 
@@ -622,11 +621,12 @@ int config_parse_many(
 
         r = conf_files_list_dropins(&files, dropin_dirname, root, conf_file_dirs);
         if (r < 0)
-                return r;
+                return log_full_errno(FLAGS_SET(flags, CONFIG_PARSE_WARN) ? LOG_WARNING : LOG_DEBUG, r,
+                                      "Failed to list up drop-in configs in %s: %m", dropin_dirname);
 
         r = config_parse_many_files(root, conf_files, files, sections, lookup, table, flags, userdata, ret_stats_by_path);
         if (r < 0)
-                return r;
+                return r; /* config_parse_many_files() logs internally. */
 
         if (ret_dropin_files)
                 *ret_dropin_files = TAKE_PTR(files);
@@ -647,22 +647,16 @@ int config_parse_standard_file_with_dropins_full(
 
         const char* const *conf_paths = (const char* const*) CONF_PATHS_STRV("");
         _cleanup_strv_free_ char **configs = NULL;
-        int r;
+        int r, level = FLAGS_SET(flags, CONFIG_PARSE_WARN) ? LOG_WARNING : LOG_DEBUG;
 
         /* Build the list of main config files */
         r = strv_extend_strv_biconcat(&configs, root, conf_paths, main_file);
-        if (r < 0) {
-                if (flags & CONFIG_PARSE_WARN)
-                        log_oom();
-                return r;
-        }
+        if (r < 0)
+                return log_oom_full(level);
 
         _cleanup_free_ char *dropin_dirname = strjoin(main_file, ".d");
-        if (!dropin_dirname) {
-                if (flags & CONFIG_PARSE_WARN)
-                        log_oom();
-                return -ENOMEM;
-        }
+        if (!dropin_dirname)
+                return log_oom_full(level);
 
         return config_parse_many(
                         (const char* const*) configs,
@@ -795,6 +789,37 @@ bool stats_by_path_equal(Hashmap *a, Hashmap *b) {
         return true;
 }
 
+int config_section_parse(
+                const ConfigSectionParser *parsers,
+                size_t n_parsers,
+                const char *unit,
+                const char *filename,
+                unsigned line,
+                const char *section,
+                unsigned section_line,
+                const char *lvalue,
+                int ltype,
+                const char *rvalue,
+                void *userdata) {
+
+        assert(parsers);
+        assert(n_parsers > 0);
+        assert(ltype >= 0);
+        assert((size_t) ltype < n_parsers);
+        assert(userdata);
+
+        const ConfigSectionParser *e = parsers + ltype;
+        assert(e->parser);
+
+        /* This is used when a object is dynamically allocated per [SECTION] in a config parser, e.g.
+         * [Address] for systemd.network. Takes the allocated object as 'userdata', then it is passed to
+         * config parsers in the table. The 'data' field points to an element of the passed object, where
+         * its offset is given by the table. */
+
+        return e->parser(unit, filename, line, section, section_line, lvalue, e->ltype, rvalue,
+                         (uint8_t*) userdata + e->offset, userdata);
+}
+
 void config_section_hash_func(const ConfigSection *c, struct siphash *state) {
         siphash24_compress_string(c->filename, state);
         siphash24_compress_typesafe(c->line, state);
@@ -854,7 +879,7 @@ int _hashmap_by_section_find_unused_line(
 }
 
 #define DEFINE_PARSER(type, vartype, conv_func)                         \
-        DEFINE_CONFIG_PARSE_PTR(config_parse_##type, conv_func, vartype, "Failed to parse " #type " value")
+        DEFINE_CONFIG_PARSE_PTR(config_parse_##type, conv_func, vartype)
 
 DEFINE_PARSER(int, int, safe_atoi);
 DEFINE_PARSER(long, long, safe_atoli);
@@ -872,7 +897,7 @@ DEFINE_PARSER(mode, mode_t, parse_mode);
 DEFINE_PARSER(pid, pid_t, parse_pid);
 
 int config_parse_iec_size(
-                const char* unit,
+                const char *unit,
                 const char *filename,
                 unsigned line,
                 const char *section,
@@ -894,17 +919,15 @@ int config_parse_iec_size(
         r = parse_size(rvalue, 1024, &v);
         if (r >= 0 && (uint64_t) (size_t) v != v)
                 r = -ERANGE;
-        if (r < 0) {
-                log_syntax(unit, LOG_WARNING, filename, line, r, "Failed to parse size value '%s', ignoring: %m", rvalue);
-                return 0;
-        }
+        if (r < 0)
+                return log_syntax_parse_error(unit, filename, line, r, lvalue, rvalue);
 
         *sz = (size_t) v;
-        return 0;
+        return 1;
 }
 
 int config_parse_si_uint64(
-                const char* unit,
+                const char *unit,
                 const char *filename,
                 unsigned line,
                 const char *section,
@@ -924,13 +947,13 @@ int config_parse_si_uint64(
 
         r = parse_size(rvalue, 1000, sz);
         if (r < 0)
-                log_syntax(unit, LOG_WARNING, filename, line, r, "Failed to parse size value '%s', ignoring: %m", rvalue);
+                return log_syntax_parse_error(unit, filename, line, r, lvalue, rvalue);
 
-        return 0;
+        return 1;
 }
 
 int config_parse_iec_uint64(
-                const char* unit,
+                const char *unit,
                 const char *filename,
                 unsigned line,
                 const char *section,
@@ -950,13 +973,13 @@ int config_parse_iec_uint64(
 
         r = parse_size(rvalue, 1024, bytes);
         if (r < 0)
-                log_syntax(unit, LOG_WARNING, filename, line, r, "Failed to parse size value, ignoring: %s", rvalue);
+                return log_syntax_parse_error(unit, filename, line, r, lvalue, rvalue);
 
-        return 0;
+        return 1;
 }
 
 int config_parse_iec_uint64_infinity(
-                const char* unit,
+                const char *unit,
                 const char *filename,
                 unsigned line,
                 const char *section,
@@ -973,14 +996,14 @@ int config_parse_iec_uint64_infinity(
 
         if (streq(rvalue, "infinity")) {
                 *bytes = UINT64_MAX;
-                return 0;
+                return 1;
         }
 
         return config_parse_iec_uint64(unit, filename, line, section, section_line, lvalue, ltype, rvalue, data, userdata);
 }
 
 int config_parse_bool(
-                const char* unit,
+                const char *unit,
                 const char *filename,
                 unsigned line,
                 const char *section,
@@ -991,24 +1014,72 @@ int config_parse_bool(
                 void *data,
                 void *userdata) {
 
-        int k;
         bool *b = ASSERT_PTR(data);
         bool fatal = ltype;
+        int r;
 
         assert(filename);
         assert(lvalue);
         assert(rvalue);
 
-        k = parse_boolean(rvalue);
-        if (k < 0) {
-                log_syntax(unit, fatal ? LOG_ERR : LOG_WARNING, filename, line, k,
-                           "Failed to parse boolean value%s: %s",
-                           fatal ? "" : ", ignoring", rvalue);
+        r = parse_boolean(rvalue);
+        if (r < 0) {
+                log_syntax_parse_error_full(unit, filename, line, r, fatal, lvalue, rvalue);
                 return fatal ? -ENOEXEC : 0;
         }
 
-        *b = k;
-        return 0;
+        *b = r;
+        return 1; /* set */
+}
+
+int config_parse_uint32_flag(
+                const char *unit,
+                const char *filename,
+                unsigned line,
+                const char *section,
+                unsigned section_line,
+                const char *lvalue,
+                int ltype,
+                const char *rvalue,
+                void *data,
+                void *userdata) {
+
+        uint32_t *flags = ASSERT_PTR(data);
+        int r;
+
+        assert(ltype != 0);
+
+        r = isempty(rvalue) ? 0 : parse_boolean(rvalue);
+        if (r < 0)
+                return log_syntax_parse_error(unit, filename, line, r, lvalue, rvalue);
+
+        SET_FLAG(*flags, ltype, r);
+        return 1;
+}
+
+int config_parse_uint32_invert_flag(
+                const char *unit,
+                const char *filename,
+                unsigned line,
+                const char *section,
+                unsigned section_line,
+                const char *lvalue,
+                int ltype,
+                const char *rvalue,
+                void *data,
+                void *userdata) {
+
+        uint32_t *flags = ASSERT_PTR(data);
+        int r;
+
+        assert(ltype != 0);
+
+        r = isempty(rvalue) ? 0 : parse_boolean(rvalue);
+        if (r < 0)
+                return log_syntax_parse_error(unit, filename, line, r, lvalue, rvalue);
+
+        SET_FLAG(*flags, ltype, !r);
+        return 1;
 }
 
 int config_parse_id128(
@@ -1031,16 +1102,18 @@ int config_parse_id128(
         assert(rvalue);
 
         r = id128_from_string_nonzero(rvalue, result);
-        if (r == -ENXIO)
+        if (r == -ENXIO) {
                 log_syntax(unit, LOG_WARNING, filename, line, r, "128-bit ID/UUID is all 0, ignoring: %s", rvalue);
-        else if (r < 0)
-                log_syntax(unit, LOG_WARNING, filename, line, r, "Failed to parse 128-bit ID/UUID, ignoring: %s", rvalue);
+                return 0;
+        }
+        if (r < 0)
+                return log_syntax_parse_error(unit, filename, line, r, lvalue, rvalue);
 
-        return 0;
+        return 1;
 }
 
 int config_parse_tristate(
-                const char* unit,
+                const char *unit,
                 const char *filename,
                 unsigned line,
                 const char *section,
@@ -1066,11 +1139,8 @@ int config_parse_tristate(
         }
 
         r = parse_tristate(rvalue, t);
-        if (r < 0) {
-                log_syntax(unit, LOG_WARNING, filename, line, r,
-                           "Failed to parse boolean value for %s=, ignoring: %s", lvalue, rvalue);
-                return 0;
-        }
+        if (r < 0)
+                return log_syntax_parse_error(unit, filename, line, r, lvalue, rvalue);
 
         return 1;
 }
@@ -1117,7 +1187,7 @@ int config_parse_string(
                 return 0;
         }
 
-        r = free_and_strdup_warn(s, empty_to_null(rvalue));
+        r = free_and_strdup_warn(s, rvalue);
         if (r < 0)
                 return r;
 
@@ -1145,7 +1215,7 @@ int config_parse_dns_name(
 
         if (isempty(rvalue)) {
                 *hostname = mfree(*hostname);
-                return 0;
+                return 1;
         }
 
         r = dns_name_is_valid(rvalue);
@@ -1160,7 +1230,11 @@ int config_parse_dns_name(
                 return 0;
         }
 
-        return free_and_strdup_warn(hostname, rvalue);
+        r = free_and_strdup_warn(hostname, rvalue);
+        if (r < 0)
+                return r;
+
+        return 1;
 }
 
 int config_parse_hostname(
@@ -1183,7 +1257,7 @@ int config_parse_hostname(
 
         if (isempty(rvalue)) {
                 *hostname = mfree(*hostname);
-                return 0;
+                return 1;
         }
 
         if (!hostname_is_valid(rvalue, 0)) {
@@ -1217,8 +1291,10 @@ int config_parse_path(
         assert(lvalue);
         assert(rvalue);
 
-        if (isempty(rvalue))
-                goto finalize;
+        if (isempty(rvalue)) {
+                *s = mfree(*s);
+                return 1;
+        }
 
         n = strdup(rvalue);
         if (!n)
@@ -1228,8 +1304,8 @@ int config_parse_path(
         if (r < 0)
                 return fatal ? -ENOEXEC : 0;
 
-finalize:
-        return free_and_replace(*s, n);
+        free_and_replace(*s, n);
+        return 1;
 }
 
 int config_parse_strv(
@@ -1239,7 +1315,7 @@ int config_parse_strv(
                 const char *section,
                 unsigned section_line,
                 const char *lvalue,
-                int ltype,
+                int ltype, /* When true, duplicated entries will be filtered. */
                 const char *rvalue,
                 void *data,
                 void *userdata) {
@@ -1253,26 +1329,29 @@ int config_parse_strv(
 
         if (isempty(rvalue)) {
                 *sv = strv_free(*sv);
-                return 0;
+                return 1;
         }
 
+        _cleanup_strv_free_ char **strv = NULL;
         for (const char *p = rvalue;;) {
-                char *word = NULL;
+                char *word;
 
                 r = extract_first_word(&p, &word, NULL, EXTRACT_UNQUOTE|EXTRACT_RETAIN_ESCAPE);
+                if (r < 0)
+                        return log_syntax_parse_error(unit, filename, line, r, lvalue, rvalue);
                 if (r == 0)
-                        return 0;
-                if (r == -ENOMEM)
-                        return log_oom();
-                if (r < 0) {
-                        log_syntax(unit, LOG_WARNING, filename, line, r, "Invalid syntax, ignoring: %s", rvalue);
-                        return 0;
-                }
+                        break;
 
-                r = strv_consume(sv, word);
+                r = strv_consume(&strv, word);
                 if (r < 0)
                         return log_oom();
         }
+
+        r = strv_extend_strv_consume(sv, TAKE_PTR(strv), /* filter_duplicates = */ ltype);
+        if (r < 0)
+                return log_oom();
+
+        return 1;
 }
 
 int config_parse_warn_compat(
@@ -1330,14 +1409,12 @@ int config_parse_log_facility(
         assert(data);
 
         x = log_facility_unshifted_from_string(rvalue);
-        if (x < 0) {
-                log_syntax(unit, LOG_WARNING, filename, line, x, "Failed to parse log facility, ignoring: %s", rvalue);
-                return 0;
-        }
+        if (x < 0)
+                return log_syntax_parse_error(unit, filename, line, x, lvalue, rvalue);
 
         *o = (x << 3) | LOG_PRI(*o);
 
-        return 0;
+        return 1;
 }
 
 int config_parse_log_level(
@@ -1360,17 +1437,15 @@ int config_parse_log_level(
         assert(data);
 
         x = log_level_from_string(rvalue);
-        if (x < 0) {
-                log_syntax(unit, LOG_WARNING, filename, line, x, "Failed to parse log level, ignoring: %s", rvalue);
-                return 0;
-        }
+        if (x < 0)
+                return log_syntax_parse_error(unit, filename, line, x, lvalue, rvalue);
 
         if (*o < 0) /* if it wasn't initialized so far, assume zero facility */
                 *o = x;
         else
                 *o = (*o & LOG_FACMASK) | x;
 
-        return 0;
+        return 1;
 }
 
 int config_parse_signal(
@@ -1393,13 +1468,11 @@ int config_parse_signal(
         assert(sig);
 
         r = signal_from_string(rvalue);
-        if (r <= 0) {
-                log_syntax(unit, LOG_WARNING, filename, line, r, "Failed to parse signal name, ignoring: %s", rvalue);
-                return 0;
-        }
+        if (r <= 0)
+                return log_syntax_parse_error(unit, filename, line, r, lvalue, rvalue);
 
         *sig = r;
-        return 0;
+        return 1;
 }
 
 int config_parse_personality(
@@ -1421,18 +1494,17 @@ int config_parse_personality(
         assert(rvalue);
         assert(personality);
 
-        if (isempty(rvalue))
-                p = PERSONALITY_INVALID;
-        else {
-                p = personality_from_string(rvalue);
-                if (p == PERSONALITY_INVALID) {
-                        log_syntax(unit, LOG_WARNING, filename, line, 0, "Failed to parse personality, ignoring: %s", rvalue);
-                        return 0;
-                }
+        if (isempty(rvalue)) {
+                *personality = PERSONALITY_INVALID;
+                return 1;
         }
 
+        p = personality_from_string(rvalue);
+        if (p == PERSONALITY_INVALID)
+                return log_syntax_parse_error(unit, filename, line, 0, lvalue, rvalue);
+
         *personality = p;
-        return 0;
+        return 1;
 }
 
 int config_parse_ifname(
@@ -1456,7 +1528,7 @@ int config_parse_ifname(
 
         if (isempty(rvalue)) {
                 *s = mfree(*s);
-                return 0;
+                return 1;
         }
 
         if (!ifname_valid(rvalue)) {
@@ -1464,11 +1536,11 @@ int config_parse_ifname(
                 return 0;
         }
 
-        r = free_and_strdup(s, rvalue);
+        r = free_and_strdup_warn(s, rvalue);
         if (r < 0)
-                return log_oom();
+                return r;
 
-        return 0;
+        return 1;
 }
 
 int config_parse_ifnames(
@@ -1493,21 +1565,15 @@ int config_parse_ifnames(
 
         if (isempty(rvalue)) {
                 *s = strv_free(*s);
-                return 0;
+                return 1;
         }
 
         for (const char *p = rvalue;;) {
                 _cleanup_free_ char *word = NULL;
 
                 r = extract_first_word(&p, &word, NULL, 0);
-                if (r == -ENOMEM)
-                        return log_oom();
-                if (r < 0) {
-                        log_syntax(unit, LOG_WARNING, filename, line, r,
-                                   "Failed to extract interface name, ignoring assignment: %s",
-                                   rvalue);
-                        return 0;
-                }
+                if (r < 0)
+                        return log_syntax_parse_error(unit, filename, line, r, lvalue, rvalue);
                 if (r == 0)
                         break;
 
@@ -1527,7 +1593,7 @@ int config_parse_ifnames(
         if (r < 0)
                 return log_oom();
 
-        return 0;
+        return 1;
 }
 
 int config_parse_ip_port(
@@ -1552,18 +1618,15 @@ int config_parse_ip_port(
 
         if (isempty(rvalue)) {
                 *s = 0;
-                return 0;
+                return 1;
         }
 
         r = parse_ip_port(rvalue, &port);
-        if (r < 0) {
-                log_syntax(unit, LOG_WARNING, filename, line, r, "Failed to parse port '%s'.", rvalue);
-                return 0;
-        }
+        if (r < 0)
+                return log_syntax_parse_error(unit, filename, line, r, lvalue, rvalue);
 
         *s = port;
-
-        return 0;
+        return 1;
 }
 
 int config_parse_mtu(
@@ -1591,11 +1654,8 @@ int config_parse_mtu(
                            rvalue);
                 return 0;
         }
-        if (r < 0) {
-                log_syntax(unit, LOG_WARNING, filename, line, r,
-                           "Failed to parse MTU value '%s', ignoring: %m", rvalue);
-                return 0;
-        }
+        if (r < 0)
+                return log_syntax_parse_error(unit, filename, line, r, lvalue, rvalue);
 
         return 1;
 }
@@ -1623,10 +1683,8 @@ int config_parse_rlimit(
                 log_syntax(unit, LOG_WARNING, filename, line, r, "Soft resource limit chosen higher than hard limit, ignoring: %s", rvalue);
                 return 0;
         }
-        if (r < 0) {
-                log_syntax(unit, LOG_WARNING, filename, line, r, "Failed to parse resource value, ignoring: %s", rvalue);
-                return 0;
-        }
+        if (r < 0)
+                return log_syntax_parse_error(unit, filename, line, r, lvalue, rvalue);
 
         if (rl[ltype])
                 *rl[ltype] = d;
@@ -1636,11 +1694,11 @@ int config_parse_rlimit(
                         return log_oom();
         }
 
-        return 0;
+        return 1;
 }
 
 int config_parse_permille(
-                const char* unit,
+                const char *unit,
                 const char *filename,
                 unsigned line,
                 const char *section,
@@ -1659,19 +1717,15 @@ int config_parse_permille(
         assert(rvalue);
 
         r = parse_permille(rvalue);
-        if (r < 0) {
-                log_syntax(unit, LOG_WARNING, filename, line, r,
-                           "Failed to parse permille value, ignoring: %s", rvalue);
-                return 0;
-        }
+        if (r < 0)
+                return log_syntax_parse_error(unit, filename, line, r, lvalue, rvalue);
 
         *permille = (unsigned) r;
-
-        return 0;
+        return 1;
 }
 
 int config_parse_vlanprotocol(
-                const char* unit,
+                const char *unit,
                 const char *filename,
                 unsigned line,
                 const char *section,
@@ -1689,20 +1743,17 @@ int config_parse_vlanprotocol(
 
         if (isempty(rvalue)) {
                 *vlan_protocol = -1;
-                return 0;
+                return 1;
         }
 
         if (STR_IN_SET(rvalue, "802.1ad", "802.1AD"))
                 *vlan_protocol = ETH_P_8021AD;
         else if (STR_IN_SET(rvalue, "802.1q", "802.1Q"))
                 *vlan_protocol = ETH_P_8021Q;
-        else {
-                log_syntax(unit, LOG_WARNING, filename, line, 0,
-                           "Failed to parse VLAN protocol value, ignoring: %s", rvalue);
-                return 0;
-        }
+        else
+                return log_syntax_parse_error(unit, filename, line, 0, lvalue, rvalue);
 
-        return 0;
+        return 1;
 }
 
 int config_parse_hw_addr(
@@ -1717,7 +1768,7 @@ int config_parse_hw_addr(
                 void *data,
                 void *userdata) {
 
-        struct hw_addr_data a, *hwaddr = ASSERT_PTR(data);
+        struct hw_addr_data *hwaddr = ASSERT_PTR(data);
         int r;
 
         assert(filename);
@@ -1726,18 +1777,14 @@ int config_parse_hw_addr(
 
         if (isempty(rvalue)) {
                 *hwaddr = HW_ADDR_NULL;
-                return 0;
+                return 1;
         }
 
-        r = parse_hw_addr_full(rvalue, ltype, &a);
-        if (r < 0) {
-                log_syntax(unit, LOG_WARNING, filename, line, r,
-                           "Not a valid hardware address, ignoring assignment: %s", rvalue);
-                return 0;
-        }
+        r = parse_hw_addr_full(rvalue, ltype, hwaddr);
+        if (r < 0)
+                return log_syntax_parse_error(unit, filename, line, r, lvalue, rvalue);
 
-        *hwaddr = a;
-        return 0;
+        return 1;
 }
 
 int config_parse_hw_addrs(
@@ -1762,7 +1809,7 @@ int config_parse_hw_addrs(
         if (isempty(rvalue)) {
                 /* Empty assignment resets the list */
                 *hwaddrs = set_free(*hwaddrs);
-                return 0;
+                return 1;
         }
 
         for (const char *p = rvalue;;) {
@@ -1770,15 +1817,10 @@ int config_parse_hw_addrs(
                 _cleanup_free_ struct hw_addr_data *n = NULL;
 
                 r = extract_first_word(&p, &word, NULL, 0);
+                if (r < 0)
+                        return log_syntax_parse_error(unit, filename, line, r, lvalue, rvalue);
                 if (r == 0)
-                        return 0;
-                if (r == -ENOMEM)
-                        return log_oom();
-                if (r < 0) {
-                        log_syntax(unit, LOG_WARNING, filename, line, r,
-                                   "Invalid syntax, ignoring: %s", rvalue);
-                        return 0;
-                }
+                        return 1;
 
                 n = new(struct hw_addr_data, 1);
                 if (!n)
@@ -1819,7 +1861,7 @@ int config_parse_ether_addr(
 
         if (isempty(rvalue)) {
                 *hwaddr = mfree(*hwaddr);
-                return 0;
+                return 1;
         }
 
         n = new0(struct ether_addr, 1);
@@ -1827,15 +1869,11 @@ int config_parse_ether_addr(
                 return log_oom();
 
         r = parse_ether_addr(rvalue, n);
-        if (r < 0) {
-                log_syntax(unit, LOG_WARNING, filename, line, r,
-                           "Not a valid MAC address, ignoring assignment: %s", rvalue);
-                return 0;
-        }
+        if (r < 0)
+                return log_syntax_parse_error(unit, filename, line, r, lvalue, rvalue);
 
         free_and_replace(*hwaddr, n);
-
-        return 0;
+        return 1;
 }
 
 int config_parse_ether_addrs(
@@ -1860,7 +1898,7 @@ int config_parse_ether_addrs(
         if (isempty(rvalue)) {
                 /* Empty assignment resets the list */
                 *hwaddrs = set_free(*hwaddrs);
-                return 0;
+                return 1;
         }
 
         for (const char *p = rvalue;;) {
@@ -1868,15 +1906,10 @@ int config_parse_ether_addrs(
                 _cleanup_free_ struct ether_addr *n = NULL;
 
                 r = extract_first_word(&p, &word, NULL, 0);
+                if (r < 0)
+                        return log_syntax_parse_error(unit, filename, line, r, lvalue, rvalue);
                 if (r == 0)
-                        return 0;
-                if (r == -ENOMEM)
-                        return log_oom();
-                if (r < 0) {
-                        log_syntax(unit, LOG_WARNING, filename, line, r,
-                                   "Invalid syntax, ignoring: %s", rvalue);
-                        return 0;
-                }
+                        return 1;
 
                 n = new(struct ether_addr, 1);
                 if (!n)
@@ -1923,15 +1956,12 @@ int config_parse_in_addr_non_null(
                         *ipv4 = (struct in_addr) {};
                 else
                         *ipv6 = (struct in6_addr) {};
-                return 0;
+                return 1;
         }
 
         r = in_addr_from_string(ltype, rvalue, &a);
-        if (r < 0) {
-                log_syntax(unit, LOG_WARNING, filename, line, r,
-                           "Failed to parse %s=, ignoring assignment: %s", lvalue, rvalue);
-                return 0;
-        }
+        if (r < 0)
+                return log_syntax_parse_error(unit, filename, line, r, lvalue, rvalue);
 
         if (!in_addr_is_set(ltype, &a)) {
                 log_syntax(unit, LOG_WARNING, filename, line, 0,
@@ -1943,7 +1973,74 @@ int config_parse_in_addr_non_null(
                 *ipv4 = a.in;
         else
                 *ipv6 = a.in6;
-        return 0;
+        return 1;
+}
+
+int config_parse_in_addr_data(
+                const char *unit,
+                const char *filename,
+                unsigned line,
+                const char *section,
+                unsigned section_line,
+                const char *lvalue,
+                int ltype,
+                const char *rvalue,
+                void *data,
+                void *userdata) {
+
+        struct in_addr_data *p = ASSERT_PTR(data);
+        int r;
+
+        assert(filename);
+        assert(lvalue);
+
+        if (isempty(rvalue)) {
+                *p = (struct in_addr_data) {};
+                return 1;
+        }
+
+        r = in_addr_from_string_auto(rvalue, &p->family, &p->address);
+        if (r < 0)
+                return log_syntax_parse_error(unit, filename, line, r, lvalue, rvalue);
+
+        return 1;
+}
+
+int config_parse_in_addr_prefix(
+                const char *unit,
+                const char *filename,
+                unsigned line,
+                const char *section,
+                unsigned section_line,
+                const char *lvalue,
+                int ltype, /* takes boolean, whether we warn about missing prefixlen */
+                const char *rvalue,
+                void *data,
+                void *userdata) {
+
+        struct in_addr_prefix *p = ASSERT_PTR(data);
+        int r;
+
+        assert(filename);
+        assert(lvalue);
+
+        if (isempty(rvalue)) {
+                *p = (struct in_addr_prefix) {};
+                return 1;
+        }
+
+        r = in_addr_prefix_from_string_auto_full(rvalue, ltype ? PREFIXLEN_REFUSE : PREFIXLEN_FULL, &p->family, &p->address, &p->prefixlen);
+        if (r == -ENOANO) {
+                r = in_addr_prefix_from_string_auto(rvalue, &p->family, &p->address, &p->prefixlen);
+                if (r >= 0)
+                        log_syntax(unit, LOG_WARNING, filename, line, r,
+                                   "%s=%s is specified without prefix length. Assuming the prefix length is %u. "
+                                   "Please specify the prefix length explicitly.", lvalue, rvalue, p->prefixlen);
+        }
+        if (r < 0)
+                return log_syntax_parse_error(unit, filename, line, r, lvalue, rvalue);
+
+        return 1;
 }
 
 int config_parse_unsigned_bounded(
@@ -1952,8 +2049,8 @@ int config_parse_unsigned_bounded(
                 unsigned line,
                 const char *section,
                 unsigned section_line,
-                const char *name,
-                const char *value,
+                const char *lvalue,
+                const char *rvalue,
                 unsigned min,
                 unsigned max,
                 bool ignoring,
@@ -1962,31 +2059,60 @@ int config_parse_unsigned_bounded(
         int r;
 
         assert(filename);
-        assert(name);
-        assert(value);
+        assert(lvalue);
+        assert(rvalue);
         assert(ret);
 
-        r = safe_atou_bounded(value, min, max, ret);
-        if (r == -ERANGE)
+        r = safe_atou_bounded(rvalue, min, max, ret);
+        if (r == -ERANGE) {
                 log_syntax(unit, LOG_WARNING, filename, line, r,
                            "Invalid '%s=%s', allowed range is %u..%u%s.",
-                           name, value, min, max, ignoring ? ", ignoring" : "");
-        else if (r < 0)
-                log_syntax(unit, LOG_WARNING, filename, line, r,
-                           "Failed to parse '%s=%s'%s: %m",
-                           name, value, ignoring ? ", ignoring" : "");
+                           lvalue, rvalue, min, max, ignoring ? ", ignoring" : "");
+                return ignoring ? 0 : r;
+        }
+        if (r < 0)
+                return log_syntax_parse_error_full(unit, filename, line, r, /* critical = */ !ignoring, lvalue, rvalue);
 
-        if (r >= 0)
-                return 1;  /* Return 1 if something was set */
-        else if (ignoring)
-                return 0;
-        else
-                return r;
+        return 1;  /* Return 1 if something was set */
 }
 
-DEFINE_CONFIG_PARSE(config_parse_percent, parse_percent, "Failed to parse percent value");
-DEFINE_CONFIG_PARSE(config_parse_permyriad, parse_permyriad, "Failed to parse permyriad value");
-DEFINE_CONFIG_PARSE_PTR(config_parse_sec_fix_0, parse_sec_fix_0, usec_t, "Failed to parse time value");
+int config_parse_calendar(
+                const char *unit,
+                const char *filename,
+                unsigned line,
+                const char *section,
+                unsigned section_line,
+                const char *lvalue,
+                int ltype,
+                const char *rvalue,
+                void *data,
+                void *userdata) {
+
+        CalendarSpec **cr = data;
+        _cleanup_(calendar_spec_freep) CalendarSpec *c = NULL;
+        int r;
+
+        assert(filename);
+        assert(lvalue);
+        assert(rvalue);
+        assert(data);
+
+        if (isempty(rvalue)) {
+                *cr = calendar_spec_free(*cr);
+                return 1;
+        }
+
+        r = calendar_spec_from_string(rvalue, &c);
+        if (r < 0)
+                return log_syntax_parse_error(unit, filename, line, r, lvalue, rvalue);
+
+        free_and_replace_full(*cr, c, calendar_spec_free);
+        return 1;
+}
+
+DEFINE_CONFIG_PARSE(config_parse_percent, parse_percent);
+DEFINE_CONFIG_PARSE(config_parse_permyriad, parse_permyriad);
+DEFINE_CONFIG_PARSE_PTR(config_parse_sec_fix_0, parse_sec_fix_0, usec_t);
 
 int config_parse_timezone(
                 const char *unit,
@@ -2009,15 +2135,48 @@ int config_parse_timezone(
 
         if (isempty(rvalue)) {
                 *tz = mfree(*tz);
-                return 0;
+                return 1;
         }
 
         r = verify_timezone(rvalue, LOG_WARNING);
-        if (r < 0) {
+        if (r < 0)
+                return log_syntax_parse_error(unit, filename, line, r, lvalue, rvalue);
+
+        r = free_and_strdup_warn(tz, rvalue);
+        if (r < 0)
+                return r;
+
+        return 1;
+}
+
+int config_parse_ip_protocol(
+                const char *unit,
+                const char *filename,
+                unsigned line,
+                const char *section,
+                unsigned section_line,
+                const char *lvalue,
+                int ltype,
+                const char *rvalue,
+                void *data,
+                void *userdata) {
+
+        uint8_t *proto = ASSERT_PTR(data);
+        int r;
+
+        r = isempty(rvalue) ? 0 : parse_ip_protocol_full(rvalue, /* relaxed= */ ltype);
+        if (r < 0)
+                return log_syntax_parse_error(unit, filename, line, r, lvalue, rvalue);
+
+        if (r > UINT8_MAX) {
+                /* linux/fib_rules.h and linux/fou.h define the netlink field as one byte, so we need to
+                 * reject protocols numbers that don't fit in one byte. */
                 log_syntax(unit, LOG_WARNING, filename, line, r,
-                           "Timezone is not valid, ignoring assignment: %s", rvalue);
+                           "Invalid '%s=%s', allowed range is 0..255, ignoring.",
+                           lvalue, rvalue);
                 return 0;
         }
 
-        return free_and_strdup_warn(tz, rvalue);
+        *proto = r;
+        return 1; /* done. */
 }
