@@ -323,6 +323,8 @@ int make_filesystem(
                 bool discard,
                 bool quiet,
                 uint64_t sector_size,
+                char *compression,
+                char *compression_level,
                 char * const *extra_mkfs_args) {
 
         _cleanup_free_ char *mkfs = NULL, *mangled_label = NULL;
@@ -422,8 +424,9 @@ int make_filesystem(
                                 "-m", "0",
                                 "-E", discard ? "discard,lazy_itable_init=1" : "nodiscard,lazy_itable_init=1",
                                 "-b", "4096",
-                                "-T", "default",
-                                node);
+                                "-T", "default");
+                if (!argv)
+                        return log_oom();
 
                 if (root && strv_extend_many(&argv, "-d", root) < 0)
                         return log_oom();
@@ -431,6 +434,9 @@ int make_filesystem(
                 if (quiet && strv_extend(&argv, "-q") < 0)
                         return log_oom();
 
+                if (strv_extend(&argv, node) < 0)
+                        return log_oom();
+
                 if (sector_size > 0) {
                         if (strv_extend(&env, "MKE2FS_DEVICE_SECTSIZE") < 0)
                                         return log_oom();
@@ -442,8 +448,7 @@ int make_filesystem(
         } else if (streq(fstype, "btrfs")) {
                 argv = strv_new(mkfs,
                                 "-L", label,
-                                "-U", vol_id,
-                                node);
+                                "-U", vol_id);
                 if (!argv)
                         return log_oom();
 
@@ -461,14 +466,12 @@ int make_filesystem(
                 if (quiet)
                         stdio_fds[1] = -EBADF;
 
-                if (sector_size > 0) {
-                        if (strv_extend(&argv, "--sectorsize") < 0)
-                                return log_oom();
+                /* mkfs.btrfs expects a sector size of at least 4k bytes. */
+                if (sector_size > 0 && strv_extendf(&argv, "--sectorsize=%"PRIu64, MAX(sector_size, 4 * U64_KB)) < 0)
+                        return log_oom();
 
-                        /* mkfs.btrfs expects a sector size of at least 4k bytes. */
-                        if (strv_extendf(&argv, "%"PRIu64, MAX(sector_size, 4 * U64_KB)) < 0)
-                                return log_oom();
-                }
+                if (strv_extend(&argv, node) < 0)
+                        return log_oom();
 
         } else if (streq(fstype, "f2fs")) {
                 argv = strv_new(mkfs,
@@ -476,8 +479,9 @@ int make_filesystem(
                                 "-f",  /* force override, without this it doesn't seem to want to write to an empty partition */
                                 "-l", label,
                                 "-U", vol_id,
-                                "-t", one_zero(discard),
-                                node);
+                                "-t", one_zero(discard));
+                if (!argv)
+                        return log_oom();
 
                 if (quiet && strv_extend(&argv, "-q") < 0)
                         return log_oom();
@@ -490,6 +494,9 @@ int make_filesystem(
                                 return log_oom();
                 }
 
+                if (strv_extend(&argv, node) < 0)
+                        return log_oom();
+
         } else if (streq(fstype, "xfs")) {
                 const char *j;
 
@@ -498,8 +505,7 @@ int make_filesystem(
                 argv = strv_new(mkfs,
                                 "-L", label,
                                 "-m", j,
-                                "-m", "reflink=1",
-                                node);
+                                "-m", "reflink=1");
                 if (!argv)
                         return log_oom();
 
@@ -538,13 +544,17 @@ int make_filesystem(
                 if (quiet && strv_extend(&argv, "-q") < 0)
                         return log_oom();
 
+                if (strv_extend(&argv, node) < 0)
+                        return log_oom();
+
         } else if (streq(fstype, "vfat")) {
 
                 argv = strv_new(mkfs,
                                 "-i", vol_id,
                                 "-n", label,
-                                "-F", "32",  /* yes, we force FAT32 here */
-                                node);
+                                "-F", "32");  /* yes, we force FAT32 here */
+                if (!argv)
+                        return log_oom();
 
                 if (sector_size > 0) {
                         if (strv_extend(&argv, "-S") < 0)
@@ -554,6 +564,9 @@ int make_filesystem(
                                 return log_oom();
                 }
 
+                if (strv_extend(&argv, node) < 0)
+                        return log_oom();
+
                 /* mkfs.vfat does not have a --quiet option so let's redirect stdout to /dev/null instead. */
                 if (quiet)
                         stdio_fds[1] = -EBADF;
@@ -565,6 +578,8 @@ int make_filesystem(
                                 "-L", label,
                                 "-U", vol_id,
                                 node);
+                if (!argv)
+                        return log_oom();
 
                 if (quiet)
                         stdio_fds[1] = -EBADF;
@@ -572,28 +587,55 @@ int make_filesystem(
         } else if (streq(fstype, "squashfs")) {
 
                 argv = strv_new(mkfs,
-                                root, node,
+                                root, node, /* mksquashfs expects its arguments before the options. */
                                 "-noappend");
+                if (!argv)
+                        return log_oom();
+
+                if (compression) {
+                        if (strv_extend_many(&argv, "-comp", compression) < 0)
+                                return log_oom();
+
+                        if (compression_level && strv_extend_many(&argv, "-Xcompression-level", compression_level) < 0)
+                                return log_oom();
+                }
 
                 /* mksquashfs -quiet option is pretty new so let's redirect stdout to /dev/null instead. */
                 if (quiet)
                         stdio_fds[1] = -EBADF;
 
         } else if (streq(fstype, "erofs")) {
-
                 argv = strv_new(mkfs,
-                                "-U", vol_id,
-                                node, root);
+                                "-U", vol_id);
+                if (!argv)
+                        return log_oom();
 
                 if (quiet && strv_extend(&argv, "--quiet") < 0)
                         return log_oom();
 
-        } else
+                if (compression) {
+                        _cleanup_free_ char *c = NULL;
+
+                        c = strjoin("-z", compression);
+                        if (!c)
+                                return log_oom();
+
+                        if (compression_level && !strextend(&c, ",level=", compression_level))
+                                return log_oom();
+
+                        if (strv_extend(&argv, c) < 0)
+                                return log_oom();
+                }
+
+                if (strv_extend_many(&argv, node, root) < 0)
+                        return log_oom();
+
+        } else {
                 /* Generic fallback for all other file systems */
                 argv = strv_new(mkfs, node);
-
-        if (!argv)
-                return log_oom();
+                if (!argv)
+                        return log_oom();
+        }
 
         if (extra_mkfs_args && strv_extend_strv(&argv, extra_mkfs_args, false) < 0)
                 return log_oom();
