@@ -10,6 +10,12 @@
 #  define ARCHITECTURE_SUPPORTS_SMBIOS 0
 #endif
 
+#if defined(__x86_64__) || defined(__i386__)
+# define ARCHITECTURE_SUPPORTS_VMGENID 1
+#else
+# define ARCHITECTURE_SUPPORTS_VMGENID 0
+#endif
+
 #if defined(__x86_64__) || defined(__arm__) || defined(__aarch64__)
 #  define ARCHITECTURE_SUPPORTS_TPM 1
 #else
@@ -32,9 +38,15 @@
 #  define DEFAULT_SERIAL_TTY "ttyS0"
 #endif
 
+#if defined(__x86_64__) || defined(__i386__)
+#  define ARCHITECTURE_SUPPORTS_HPET 1
+#else
+#  define ARCHITECTURE_SUPPORTS_HPET 0
+#endif
+
 #if defined(__x86_64__) || defined(__i386__)
 #  define QEMU_MACHINE_TYPE "q35"
-#elif defined(__arm__) || defined(__aarch64__) || defined(__riscv) || defined(__loongarch64)
+#elif defined(__arm__) || defined(__aarch64__) || defined(__riscv) || defined(__loongarch64) || defined(__m68k__)
 #  define QEMU_MACHINE_TYPE "virt"
 #elif defined(__s390__) || defined(__s390x__)
 #  define QEMU_MACHINE_TYPE "s390-ccw-virtio"
@@ -42,8 +54,10 @@
 #  define QEMU_MACHINE_TYPE "pseries"
 #elif defined(__mips__)
 #  define QEMU_MACHINE_TYPE "malta"
+#elif defined(__sparc__)
+#  define QEMU_MACHINE_TYPE "sun4u"
 #else
-#  error "No qemu machine defined for this architecture"
+#  define QEMU_MACHINE_TYPE "none"
 #endif
 
 typedef struct OvmfConfig {
