@@ -5,7 +5,7 @@ set -eux
 set -o pipefail
 
 if [[ $(systemctl is-enabled systemd-udev-load-credentials.service) == not-found ]]; then
-    echo "Missing systemd-udev-load-credentials.service" >>/skipped
+    echo "Missing systemd-udev-load-credentials.service"
     exit 0
 fi
 
